/**
 * This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Iacopo Papalini <ipapalini@domotz.com> on 24/04/15.
 *
 */

var startSshTunnel = function (message, resourceLocator) {

    var MODULE = 'startSshTunnel - ';

    var ssh = resourceLocator.networkTools.ssh;

    var payload = message.payload;

    var deviceHost = payload.device.host;
    var devicePort = payload.device.port;
    var user = payload.server.user;
    var host = payload.server.host;
    var port = payload.server.port;
    var privateListeningPort = payload.server.private_listening_port;
    var ssh_max_retries = payload.ssh_connection_max_retries || 10;
    var ssh_retry_interval = payload.ssh_connection_retry_interval || 1000;

    var my_console = resourceLocator.log.decorateLogs(MODULE);

    var async = resourceLocator.async;

    var options = {
        user: user,
        password: payload.server.password,
        private_key: payload.server.private_key,
        host: host,
        port: port,
        remote_port_forwarding: (payload.server.private_listening_iface || '0.0.0.0') + ':' + privateListeningPort + ':' + deviceHost + ':' + devicePort,
        disable_remote_command: true,
        ttl: payload.ttl * 1000,
        log_prefix: message.correlationId,
        host_key_checking: 'no',
        server_alive_interval: 60,
        server_alive_count_max: 5
    };

    function describeTunnel() {
        return [user, '@', host, ':', port, ' ', privateListeningPort, ':',
            deviceHost, ':', devicePort].join('');
    }

    return {
        describe: function () {
            // 'cid - startSshTunnel - domotz@zbx01.cloudserver.it:22 8080:192.168.5.64:5000'
            return MODULE + describeTunnel();
        },
        execute: function (callback) {
            var ERR = 'E',
                SUCCESS = null;

            function sshConnect(retry_cb) {
                var failed = false;

                var optionsCopy = JSON.parse(JSON.stringify(options));

                ssh.exec(null, optionsCopy).catch(function () {
                    failed = true;
                });

                setTimeout(function () {
                    if (failed) {
                        my_console.info("ssh process exited after one second");
                        retry_cb(ERR);
                    } else {
                        my_console.info("ssh process is up after one second");
                        retry_cb(SUCCESS);
                    }
                }, ssh_retry_interval)
            }

            async.retry({times: ssh_max_retries}, sshConnect, function (err) {
                if (err) {
                    my_console.error("ssh process cannot start, close the RPC call with failure");
                    callback({success: false});
                    return;
                }
                my_console.info("ssh process started, close the RPC call with success");
                callback({success: true})
            });
        }
    };
};


module.exports.command = startSshTunnel;