/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Tommaso Latini <tommaso@domotz.com> on 09/03/16.
 */

const MODULE = 'SIGNAL_HANDLER - ';

var assert = require('assert');

var setHandler = function (proc, log, status, eventEntity, resourceLocator) {
    var my_console = log.decorateLogs(MODULE);

    function flushLog() {
        if (log) {
            log.flush();
        }
    }

    function initShutdown(reason, detail) {
        if (status && status.isConfigured()) {
            try {
                if (reason === 'uncaughtException')
                    signalUnhandledError(detail);
            } catch (e) {
                my_console.error('An error happened while signaling unhandled error:');
                my_console.error(e.stack);
            }
        }
        proc.gracefulShutdown();
    }

    function signalUnhandledError(detail) {
        if (!status || !status.isConfigured()) {
            return;
        }
        detail = detail || '';
        detail = require('querystring').escape(detail.trim()).slice(0, 1000);
        var configuration = resourceLocator.configuration;
        var agentUrl = configuration.engine.api_endpoints.agent.replace(/\/(hub|agent)-api\//, '/agent-crash-report-api/') + 'agent/' + configuration.id;
        require('request').get(agentUrl + '?detail=' + detail);
    }

    process.on('uncaughtException', function (error) {
        if (error instanceof assert.AssertionError) {
            my_console.warn('Ignoring failed Assertion: %s', error.message);
        } else {
            my_console.error('Uncaught exception. Message: %s, code: %s', error.message, error.code);
            my_console.error(error.stack);
            var traceback = error.stack.split('\n');
            initShutdown('uncaughtException', traceback[0] + ' ' + traceback[1]);
        }
    });

    process.on('SIGHUP', function () {
        my_console.warn('Catch signal SIGHUP');
        flushLog();
    });

    process.on('SIGINT', function () {
        my_console.warn('Catch signal SIGINT');
        initShutdown('sigint');
    });

    process.on('SIGTERM', function () {
        my_console.warn('Catch signal SIGTERM');
        initShutdown('sigterm');
    });

};

module.exports.setHandler = setHandler;