/**
 * This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Alessio Sanfratello <asanfratello@domotz.com> on 16/06/17.
 */

const MODULE = 'MODULES';


function getAvailableModules(resourceLocator, callback) {
    var jamDriver = resourceLocator.jamDriver;
    var mlab = resourceLocator.networkTools.mlabSpeedTest;
    var modules = {};

    if (resourceLocator.netPing) {
        modules.netping = {
            version: ""
        }
    }

    if (mlab.isAvailable()) {
        modules.mlab = {
            version: ""
        };
    }

    if (jamDriver.hasOwnProperty('getFingVersion')) {
        jamDriver.getFingVersion(function (error, message) {
            if (error) {
                callback(modules);
            } else {
                modules.fing = {
                    version: message
                };
                callback(modules);
            }
            console.info('%s - Available modules: %s', MODULE, JSON.stringify(modules));
        });

    } else {
        if (jamDriver.hasOwnProperty('luxulApiVersion')) {
            modules.luxul_blocking_api = {
                version: jamDriver.luxulApiVersion().toString()
            };
        }
        console.info('%s - Available modules: %s', MODULE, JSON.stringify(modules));
        callback(modules);
    }
}

module.exports = {
    getAvailableModules: getAvailableModules
};